// Definitions for the automated inclusion into the parser
// LABEL:		MMEnzyme
// CLASSNAME:	ProteinMMEnzyme
// 
#ifndef ProteinMMEnzyme_H
#define ProteinMMEnzyme_H

#include <string>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <math.h>
#include "random.h"
#include "protein.h"
#include "pool.h"
#include "parameter.h"


//! Class that model a simple enzyme

/*!
This enzyme takes up a substrate, converts it to a product, and releases the product.

We need: two pools and three rates (binding, conversion, release)
*/ 

class ProteinMMEnzyme : public Protein
{
	
	public:
	
	//! Constructor
	ProteinMMEnzyme(const std::string name, const double &timestep);
	//! Destructor
	~ProteinMMEnzyme() { }	
	
	//! Parser for setup line
	bool parseLine(char *linep);
	
	
	
	private:
	
	//! @name Internal functions (reactions) of the protein
	//@{
	//! The central reaction: take up particles and release them to the other pool
	void reaction_A1(Protein *p);
	void reaction_R1(Protein *p);
	void reaction_D1(Protein *p);
	//@}
	
	//! Verify that all neccessary pools are defined
	void verify_pools();
	
	void initInternalOutput(const std::string fileName) { }
	std::string listInternals();
	void writeInternals();
	
	//! @name Pointers to the two pools and variables for the uptake constants
	//@{
	Pool *SubstratePoolp, *ProductPoolp;
	double kon_S, koff_P, k_conv;
	//@}
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char SubstratePool[MAX_POOLNAME_LENGTH], 
		 ProductPool[MAX_POOLNAME_LENGTH];
	//@}
		 
	// Defs of the internal registers and binding sites.
	bool bs_S, bs_P;
	
	//! Timer variables for some of the reactions
	unsigned long int timer_R1, timer_D1;
};

#endif /* ProteinMMEnzyme_H */
