// Definitions for the automated inclusion into the parser
// LABEL:		MMEnzyme2
// CLASSNAME:	ProteinMMEnzyme2
// 
#ifndef ProteinMMEnzyme2_H
#define ProteinMMEnzyme2_H

#include <string>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <math.h>
#include "random.h"
#include "protein.h"
#include "pool.h"
#include "parameter.h"


//! Class that model a simple enzyme that need ATP for the conversion

/*!
This enzyme takes up a substrate and ATP, converts it to a product, and releases the product and ADP.
ADP actually means "ADP + Pi" => phosphate is ignored

We need: four pools and five rates (2*binding, conversion, 2*release)
*/ 

class ProteinMMEnzyme2 : public Protein
{
	
	public:
	
	//! Constructor
	ProteinMMEnzyme2(const std::string name, const double &timestep);
	//! Destructor
	~ProteinMMEnzyme2() { }	
	
	//! Parser for setup line
	bool parseLine(char *linep);
	
	
	
	private:
	
	//! @name Internal functions (reactions) of the protein
	//@{
	//! Association 1: uptake of the substrate
	void reaction_A1(Protein *p);
	//! Association 2: ATP binding (only after substrate binding)
	void reaction_A2(Protein *p);
	//! the conversion itself: S + ATP => P + ADP
	void reaction_R1(Protein *p);
	//! Dissociation 1: product release
	void reaction_D1(Protein *p);
	//! Dissociation 2: ADP release (independent of product release)
	void reaction_D2(Protein *p);
	//@}
	
	//! Verify that all neccessary pools are defined
	void verify_pools();
	
	void initInternalOutput(const std::string fileName) { }
	std::string listInternals();
	void writeInternals();
	
	//! @name Pointers to the two pools and variables for the uptake constants
	//@{
	Pool *SubstratePoolp, *ProductPoolp, *ATPPoolp, *ADPPoolp;
	double kon_S, koff_P, k_conv, kon_ATP, koff_ADP;
	//@}
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char SubstratePool[MAX_POOLNAME_LENGTH], 
		 ProductPool[MAX_POOLNAME_LENGTH],
		 ATPPool[MAX_POOLNAME_LENGTH],
		 ADPPool[MAX_POOLNAME_LENGTH];
	//@}
		 
	// Defs of the internal registers and binding sites.
	bool bs_S, bs_P, bs_ATP, bs_ADP;
	
	//! Timer variables for some of the reactions
	unsigned long int timer_R1, timer_D1, timer_D2;
};

#endif /* ProteinMMEnzyme2_H */
