// Definitions for the automated inclusion into the parser
// LABEL:		LHC
// CLASSNAME:	ProteinLHC
// 
#ifndef ProteinLHC_h
#define ProteinLHC_h

#include<string>
#include<stdlib.h>
#include<iostream>
#include<string>
#include"random.h"
#include"protein.h"
#include"pool.h"
#include"parameter.h"

#define MAX_POOLNAME_LENGTH 128


//! Class that represents the Light Harvesting Complexes.

/*!
The LHC is a composite of one LH1 and three associated LH2 which absorbs photons 
and converts them into electronic excitations (excitons) for use in the RC.
It has no binding site.

To model the decay of excitons the LHC can take excitons out of the pool, too.

The LHC accesses two Pools: Light, Excitons
*/

class ProteinLHC : public Protein
{
	public:
	
	//! Constructor, sets the initializing states of the binindg, sites, internals ,...      
	ProteinLHC(const std::string name, const double &timestep);
	//! Destructor
	~ProteinLHC() { }
	
	//! @name Methods to name the pools (hard coded lengths, checks for buffer overflows)
	//@{
	// void setLightPool(char *poolName)   { strncpy(LightPool,   poolName, (MAX_POOLNAME_LENGTH - 1)); }
	// void setExcitonPool(char *poolName) { strncpy(ExcitonPool, poolName, (MAX_POOLNAME_LENGTH - 1)); }
	//@}
	
	
	//! Parse the setup line
	bool parseLine(char *linep);
	
	
	private:
	//! @name Internal functions (reactions) of the protein
	//@{
	//! Delete an exciton (= exciton decay in the LHCs)
	void reaction1(Protein *p);
	//! Capture a photon and release it as an exciton. Rate is slowed 
	//! down exponentially, when more excitons are already in the pool (= bleaching)
	void reaction2(Protein *p);
	//@}
	
	// Defs of the internal registers and binding sites.
	// -> LHC has no interals or binding sites
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char LightPool[MAX_POOLNAME_LENGTH], 
		 ExcitonPool[MAX_POOLNAME_LENGTH];
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();
	
	//! @name Pointers to pools and variables for reaction constants to avoid their repeated lookup
	//@{
	Pool *LightPoolp, *ExcitonPoolp;
	double N0_Exciton, Light_kon, Exciton_kon;
	//@}
};
#endif
