// Wrappers for the GSL random numbers

#include "random.h"


// needs to be declared here to avoid duplicates
gsl_rng *gsl_random_gen;


// init from time()
void init_ran()
{
	unsigned long int zeit;
	zeit = (unsigned long int)(time(0L));
	
	// allocate the RNG
	gsl_random_gen = gsl_rng_alloc(RNG_TYPE);
	
	// seed it with the time
	gsl_rng_set(gsl_random_gen, zeit);
}


// init with given seed
void set_ran(unsigned long int &seed)
{
	// allocate the RNG
	gsl_random_gen = gsl_rng_alloc(RNG_TYPE);
	
	// seed it with the specified seed
	gsl_rng_set(gsl_random_gen, seed);
}


// clean up again
void exit_ran() {
	gsl_rng_free(gsl_random_gen);
}



// random numbers from [0,1)
double lin_ran() {
	return(gsl_rng_uniform(gsl_random_gen));
}


// exponentially distributed numbers acc to exp(-t/t0)
double exp_ran(double &t0) {
	return(gsl_ran_exponential(gsl_random_gen, t0));
}


// normally distributed: exp(-(t-t0)^2 / (2*sigma^2))
double normal_ran(double &t0, double &sigma) {
	return(gsl_ran_gaussian(gsl_random_gen, sigma) + t0);
}

// integers from [0, n-1]
unsigned long int int_ran(unsigned long int n) {
	return(gsl_rng_uniform_int (gsl_random_gen, n));
}
